/* ENVIRON.H - Contains miscellaneous definitions of things which differ
               between Linux, OS/9 and MS-DOS

               Linux environment
               -----------------
*/

#include <stdio.h>
#include <dirent.h>
#include <time.h>

#ifndef MSDOS
#define MSDOS         "MS-DOS"
#endif

#define PRINTER       "lpt2:"
#define FORM_FEED     0x0c

#define    BLANK      ' '

#define MAXLINE       161   /* Max length of a printline including
                               formatting characters */
#define TABSPACE      8
#define NAMELTH       256   /* Max length of a path name */

#ifndef TRUE
#define     TRUE      1
#endif
#ifndef FALSE
#define     FALSE     0
#endif

/* CONSTANTS */
/* ========= */

/* Reply values returned by argparse (getopt, getarg) */

#ifndef EOF
#       define EOF    (-1)
#endif

#ifndef NULL
#       define NULL   (0)
#endif

#define FOUND         1
#define NOTHING       0

/* Constants used by autodate and autotime */

#define AUTOFMT       0
#define DDMMMYY       1
#define MANUAL        2

/* Constants for diracc (diropen, dirclose, dirread) */

#ifdef OS9
#define NAMESIZE      29       /* Size of a name in a directory entry   */
#define DIRENT        32       /* Total size of a directory entry       */
#define PATHSEP       "/"
#define SEPCHAR       '/'
#endif

#ifdef MSDOS
#define NAMESIZE      12       /* Max size of a DOS file name           */
#define ATTRIBUTE     11       /* File attribute byte offset            */
#define DIRENT        32       /* Total size of a directory entry       */
#define PATHSEP       "\\"
#define SEPCHAR       '\\'
#endif

#ifdef LINUX
#define NAMESIZE      1024     /* Max size of a Linux file name         */
#define PATHSEP       "/"
#define SEPCHAR       '/'
#endif

#define DIR_EOF       0
#define DIR_NULL      1
#define DIR_FILE      2
#define DIR_SUBDIR    3

#define FIRSTCALL     1
#define NEXTCALL      2

/* Constants used by fixext, fixname, fixpath */

#ifdef OS9
#define MAXPATH       512
#define MAXNAME       28
#define MAXFILE       26
#define MAXEXT        26
#define SEPARATOR     '/'
#endif

#ifdef MSDOS
#define MAXPATH       80
#define MAXNAME       12
#define MAXFILE       9
#define MAXEXT        5
#define SEPARATOR     '\\'
#endif

#ifdef LINUX
#define MAXPATH       1024
#define MAXNAME       512
#define MAXFILE       256
#define MAXEXT        256
#define SEPARATOR     '/'
#endif

#define DOT           '.'
#define UNDERSCORE    '_'

/* Constants for logmsg (logsetup, logmsg, logdump) */

#define BUFFLIM       21

#define DIRECT_LOG    1
#define INDIRECT_LOG  0

#define STDOUT_LOG    1
#define STDERR_LOG    2

/* Constants for setpause */

#define P_RECORD      1
#define P_PAUSE       2
#define P_NOPAUSE     3
#define P_RESTORE     4

/* Constants for tempstore (tsopen, tsclose, tsputc, 
                            tsgetc, tsputstr, tsgetstr)
*/

#define TSLIM 16384


/* TYPE DEFINITIONS */
/* ================ */

/* Used by autodate and autotime */

typedef char* CHARDATE;
typedef char* CHARTIME;

/* used by diracc (diropen, dirclose, dirread) */

typedef unsigned char BYTE;

#ifdef OS9
typedef struct {
                    FILE    *p;
                    char    path[NAMELTH];
               }
                    DIRPATH;
#endif

#ifdef MSDOS
typedef struct {
                    struct  ffblk *p;
                    int     status;
                    char    path[NAMELTH];
               }
                    DIRPATH;

#endif

#ifdef LINUX
typedef struct {
                    DIR    *p;
                    char   path[NAMESIZE];
               }
                    DIRPATH;
#endif

/* Used by tempstore (tsopen, tsclose, tsputc, tsgetc, tsputstr, tsgetstr) */

typedef struct {
                    char  *buffer;
    	            int   fileused;
    	            char  *filename;
    	            FILE* fil;
    	            int   inptr;
    	            int   outptr;
    	            int   size;
               }
                    tempstore;

    
/* GLOBAL VARIABLES */
/* ================ */

/* Macro definitions */

#ifndef min
#define min(a,b) ( a<b ? a : b )
#endif

#ifndef max
#define max(a,b) ( a>b ? a : b )
#endif

/* Variables used by the extended version of 'getopt' */

extern char *optstart;          /* string containing valid option markers */
extern int  optbad;             /* contents returned for a bad option */
extern int  optchar;            /* char beginning a given argument */
extern int  opterrfd;           /* file no for error messages */
extern int  optmaybe;           /* marker for optional arguments */
extern int  optneed;            /* marker for mandatory arguments */
extern int  optcase;            /* controls option case sensitivity */

/*      Global variables used by all versions of 'getopt' */

extern char *optarg;            /* argument of the option */
extern int  opterr;             /* set to 0 to suppress error messages */
extern int  optind;             /* index of current argv[] */
extern int  optopt;             /* the actual option */

/* FUNCTIONS */
/* ========= */

/* Function templates */

int            argval(char *s, int d);
CHARDATE       autodate(struct tm *tbp, int fmt, int code);
CHARTIME       autotime(struct tm *tbp);
char           *capitalise(char *s);
int            chopinsert(void **Table,
                          int *Items,
                          int Tablimit,
                          void *NewItem,
                          int InsertPoint);
void           *chopsearch(void *Target,
                           void **Table,
                           int Items,
                           int *InsertHere,
                           int (*Compare)(const void*, const void*));
DIRPATH        *diropen(char *p);
int            dirclose(DIRPATH *dp);
int            dirread(DIRPATH *dp, char *name);
void           dumphex(FILE *f, char *buff, int lth);
void           error(char *f, char *e, char *p);
long           filesize(char *name);
char*          fixext(char *name, char *ext);
char*          fixname(char *name, char *ext, char *type);
char*          fixpath(char *path, char *ext, char *type);
int            getarg(int nargc, char **nargv);
char           getdelcode(void);
char           getkey(void);
int            getopt(int nargc,
                      char * const nargv[],
                      const char *ostr);
char*          getpath(int argc, char *argv[], FILE *pathlist, int listflag);
char*          getwd(char * buf);
char*          index(char* s, char c);
void           logsetup(int r, int f);
void           logmsg(char* fmt, ...);
void           logdump(void);
char           *progname(void);
int            question(char *prompt);
int            setpause(int action, int pathno);
void           setprog(char *name);
char           *strmfe(char *fullname, char *name, char *ext);
int            swapdir(char *path);
void           tabclear(char *tabstops);
int            tabdefault(void);
int            tabexpand(char dest[], int i, char *tabstops, char fill);
void           tabnormal(char *tabstops);
int            tabout(FILE *fn, int i, char *tabstops);
int            tabpos(int col, char *tabstops);
void           tabset(int col, char state, char *tabstops);
void           tabspread(int gap, char *tabstops);
tempstore*     tsopen(long s, int d);
tempstore*     tsclose(tempstore *ptr);
int            tsputc(tempstore *ptr, int c);
int            tsgetc(tempstore *ptr);
int            tsputstr(tempstore *ptr, char *str);
int            tsgetstr(tempstore *ptr, char *str, int n);