/* CHOPSEARCH.C    - chopsearch() declaration */

#include "environ.h"

#ifdef OS9
#include <strings.h>
#else
#include <string.h>
#endif

/* #define CHOPTEST 10 */

/*
    chopsearch() returns a pointer to the target item if it was found
                 and NULL if it wasn't found.
*/
void *chopsearch(void *Target,
                 void **Table,
                 int Items,
                 int *InsertHere,
                 int (*Compare)(const void*, const void*))
{
    int base;
    int mid;
    int top;
    int cmp;
    void *reply;

#ifdef CHOPTEST
    int limit;
#endif

    base = 0;
    top = Items - 1;
    mid = top / 2;
    cmp = -1;

#ifdef CHOPTEST
    fprintf(stderr, "Search for %s starting at %d\n", Target, mid);
    limit = 0;
#endif

    while (cmp && base <= top)
    {

#ifdef CHOPTEST
        if(++limit > CHOPTEST)
        {
            fprintf(stderr, "Search loop?\n");
            exit(1);
        }
#endif

#ifdef CHOPTEST
        fprintf(stderr, "Search: top=%d mid=%d base=%d\n", top, mid, base);
#endif
        mid = (base + top) / 2;
        cmp = Compare(Target, Table[mid]);
        if (cmp < 0)
            top  = mid - 1;
        else
            base = mid + 1;
    }

    if (cmp == 0)
    {
        *InsertHere = mid;
        reply = Table[mid];
    }
    else
    {
        reply = (void*)NULL;
        if (cmp < 0)
            *InsertHere = mid;
        else
            *InsertHere = mid + 1;
    }

#ifdef CHOPTEST
    if (reply == NULL)
        fprintf(stderr, "Search miss: search ended at %d; insert %s at %d\n", 
                        mid, Target, *InsertHere);
    else
        fprintf(stderr, "Search found %s at %d\n", Target, mid);
#endif

    return(reply);
}
