/* TEMPSTORE.C - Temporary storage buffer.
                 Held in memory unless bigger than
                 a predefined limit, when it is put in a
                 temporary file.
*/

#include "environ.h"

#ifdef OS9
#include <strings.h>
char* mktemp();
#endif

#ifdef MSDOS
#include <string.h>
#include <stdlib.h>
#include <dir.h>
#endif

#ifdef LINUX
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#endif

/* initialise the storage system */

tempstore* tsopen(long s, int d)
{
    tempstore*    hddr;

    hddr = (tempstore*)malloc(sizeof(tempstore));
    if (hddr == NULL)
        return (tempstore*)(NULL);

    if (s == 0)
        s = TSLIM;
    hddr->buffer = NULL;
    hddr->filename = NULL;
    hddr->fil = NULL;
    hddr->size = s;
    hddr->inptr = 0;
    hddr->outptr = 0;

    if (s<=TSLIM)
    {
        hddr->fileused = FALSE;
        hddr->buffer = (char*)malloc(s);
        if (hddr->buffer == NULL)
            return (tsclose(hddr));
    }
    else
    {
        hddr->fileused = TRUE;
        hddr->filename = (char*)malloc(9);
        if (hddr->filename == NULL)
            return (tsclose(hddr));
        strcpy(hddr->filename,"TSXXXXXX");
        hddr->filename = mktemp(hddr->filename);
        if (hddr->filename == NULL)
            return (tsclose(hddr));
        if (d)
            printf("Temp store name is %s\n",hddr->filename);
        hddr->fil = fopen(hddr->filename,"w+");
        if (hddr->fil == NULL)
            return (tsclose(hddr));
    }
    return (hddr);
}

/* close the storage system */

tempstore* tsclose(tempstore *ptr)
{
    if (ptr == NULL)
        return (NULL);
    if (ptr->fileused)
    {
        if (ptr->fil != NULL)
            fclose(ptr->fil);
        if (ptr->filename != NULL)
            unlink(ptr->filename);
    }
    if (ptr->buffer != NULL)
        free(ptr->buffer);
    if (ptr->filename != NULL)
        free(ptr->filename);
    free(ptr);
    return (NULL);
}

/* store a character */

int tsputc(tempstore *ptr, int c)
{
    int    r;
    
    if (ptr == NULL)
        return (FALSE);
    if (ptr->inptr < ptr->size)
    {
        if (ptr->fileused)
            r = putc(c,ptr->fil);
        else
            r = ptr->buffer[ptr->inptr] = c;
        ptr->inptr += 1;
    }
    else
        r = !c;    
    return ( r == c );
}

/* get a character from storage */

int tsgetc(tempstore *ptr)
{
    int c;
    
    if (ptr == NULL)
        return (EOF);
    if (ptr->outptr >= ptr->inptr)
        return (EOF);
    if (ptr->fileused)
    {
        if (ptr->outptr == 0)              /* first read on the file */
        {
            fflush(ptr->fil);
            if (fseek(ptr->fil,0,0) != 0)  /* beginning of file */
                return(EOF);               /* error trap */
        }
        c = getc(ptr->fil);
    }
    else
        c = ptr->buffer[ptr->outptr];
    ptr->outptr += 1;
    return (c);
}

/* store a string */

int tsputstr(tempstore *ptr, char *str)
{
    int r;
    
    r = TRUE;
    while (*str && r)
    {
        r = tsputc(ptr,*str);
        str++;
    }
    if (r)
        r = tsputc(ptr,0);
    return (r);
}

/* get a string from storage */

int tsgetstr(tempstore *ptr, char *str, int n)
{
    int i;
    int c;
    
    i = 0;
    while ((c = tsgetc(ptr)) != 0)
    {
        if (c == EOF)
            break;
        str[i++] = c;
        if (i>=n)
            break;
    }
    if (i<n)
        str[i] = 0;
    return (c != EOF);
}
